use std::{collections::HashMap, fs, path::PathBuf, time::Duration};

use chrono::{DateTime, Local};
use mime::Mime;
use mime_sniffer::MimeTypeSnifferExt;

#[derive(Debug, Clone)]
struct CachedFile {
    pub die: DateTime<Local>,
    pub data: Vec<u8>,
    pub mime: Mime,
}

#[derive(Debug, Clone)]
pub struct FileGetter {
    cached_files: HashMap<PathBuf, CachedFile>
}

impl FileGetter {
    pub fn new() -> Self {
        Self {
            cached_files: HashMap::new()
        }
    }

    pub fn get_file<T: Into<PathBuf>>(&mut self, file: T) -> Result<Option<(Vec<u8>, Mime)>, Box<dyn std::error::Error>> {
        let file_path: PathBuf = file.into();
        if let Some(file) = self.cached_files.get_mut(&file_path) {
            file.die = Local::now() + Duration::from_secs(15);

            Ok(Some((file.data.clone(), file.mime.clone())))
        } else {
            let mut file_path = file_path;

            if file_path.is_dir() { file_path = file_path.join("index.html"); }
            if ! file_path.is_file() { return Ok(None) }

            let buf = fs::read(&file_path)?;
            let mime = buf.sniff_mime_type_ext().unwrap_or(mime_guess::from_path(&file_path).first_or(mime::APPLICATION_OCTET_STREAM));

            if self.cached_files.len() < 100 {
                self.cached_files.insert(file_path, CachedFile {
                    die: Local::now() + Duration::from_secs(15),
                    data: buf.clone(),
                    mime: mime.clone()
                });
            }

            Ok(Some((buf, mime)))
        }
    }
}