use std::future::Future;
use std::net::SocketAddr;
use std::pin::Pin;
use std::sync::Arc;

use http_body_util::Full;
use hyper::{Response, http::response::Builder};
use hyper::{service::Service, Request};
use hyper::body::{Bytes, Incoming as IncomingBody};
use std::sync::Mutex;

use crate::env::Env;
use crate::filegetter::FileGetter;

#[derive(Debug)]
pub struct SniWebsitesService {
    pub client_ip: Option<SocketAddr>,
    pub env: Env,
    pub files: Arc<Mutex<FileGetter>>
}

impl SniWebsitesService {
    fn make_body<T: Into<Bytes>>(res: T) -> Full<Bytes> {
        Full::new(res.into())
    }
}

fn res_builder() -> Builder {
    Response::builder()
        .header("Server", "blek! CC 1.0")
}

fn check_uri(uri: String) -> bool {
    if ! uri.starts_with('/') { return false }

    for seg in uri.split("/") {
        if seg.starts_with(".") {
            return false
        }
    }

    true
}

fn serve_file(sni: String, req: Request<IncomingBody>, env: &Env, files: Arc<Mutex<FileGetter>>) -> Response<Full<Bytes>> {
    let docpath = env.get_doc_for_sni(sni);
    let uri = req.uri().path().to_string();

    if ! check_uri(uri.to_string()) {
        return res_builder()
            .status(400)
            .body(SniWebsitesService::make_body("illegal URI"))
            .unwrap()
    }

    let path = docpath.join(uri.trim_start_matches('/'));
    let file = files.lock().unwrap().get_file(path).unwrap();

    if let Some(file) = file {
        res_builder()
            .header("Content-Type", file.1.to_string())
            .body(SniWebsitesService::make_body(file.0))
            .unwrap()
    } else {
        res_builder()
            .header("Content-Type", "text/html")
            .body(SniWebsitesService::make_body("<h1>not found!</h1><p>this file is not found</p>"))
            .unwrap()
    }
}

impl Service<Request<IncomingBody>> for SniWebsitesService {
    type Response = Response<Full<Bytes>>;
    type Error = hyper::Error;
    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>> + Send>>;

    fn call(&self, req: Request<IncomingBody>) -> Self::Future {
        let res = 
            if let Some(sni) = req.headers().get("x-forwarded-host") {
                let sni = sni.to_str().unwrap();
                let sni = sni.replace(&(".".to_string() + &self.env.tld), "");
                
                serve_file(sni, req, &self.env, self.files.clone())
            } else {
                res_builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Self::make_body("refusing to answer to a non proxied request"))
                    .unwrap()
            };
    
        Box::pin(async { Ok(res) })
    }
}